@@ -128,9 +128,10 @@ all app state variables as a dict is determining the method :meth:`~MainBaseApp.
 into the :ref:`config-files` for the next app run via the method :meth:`~MainBaseApp.save_app_states` - this could be
 done e.g. after the app state has changed or at least on quiting the application.
 
-changed app state values have to be saved always by calling the method :meth:`~MainBaseApp.change_app_state` (which
-ensures (1) the propagation to any duplicated (observable/bound) framework property and (2) the event notification to
-related the main app instance method).
+always call the method :meth:`~MainBaseApp.change_app_state` to change an app state value to ensure:
+
+    (1) the propagation to any duplicated (observable/bound) framework property and
+    (2) the event notification of the related (optionally declared) main app instance method.
 
 
 .. _app-state-constants:
@@ -158,15 +159,15 @@ app state events
 there are three types of notification events get fired in relation to the app state variables, using the method names:
 
 * `on_<app_state_name>`: fired if the user of the app is changing the value of an app state variable.
-* `on_<app_state_name>_save`: fired if a app state gets saved to the config file.
-* `on_app_state_version_upgrade`: fired if the user upgrades an previously installed app to a higher version.
+* `on_<app_state_name>_save`: fired if an app state gets saved to the config file.
+* `on_app_state_version_upgrade`: fired if the user upgrades a previously installed app to a higher version.
 
 the method name of the app state change notification event consists of the prefix ``on_`` followed by the variable name
 of the app state. so e.g. on a change of the `font_size` app state the notification event `on_font_size` will be
 fired/called (if exists as a method of the main app instance). these events don't provide any event arguments.
 
 the second event gets fired for each app state value just after the app states getting retrieved from the app class
-instance and before they get stored into the main config file. the method name of this event includes also the name of
+instance, and before they get stored into the main config file. the method name of this event includes also the name of
 the app state with the suffix `_save`, so e.g. for the app state `flow_id` the event method name will result in
 :meth:`on_app_state_flow_id_save`. this event is providing one event argument with the value of the app state. if the
 event method returns a value that is not `None` then this value will be stored/saved.
@@ -237,7 +238,7 @@ if in contrary either the flow change confirmation event handler exists and does
 after the flow id/path change confirmation the method :meth:`~MainAppBase.change_flow` checks if the optional
 event_kwargs key `changed_event_name` got specified and if yes then it calls this method.
 
-finally if a confirmed flow change results in a `'focus'` flow action then the event `on_flow_widget_focused` will be
+finally, if a confirmed flow change results in a `'focus'` flow action then the event `on_flow_widget_focused` will be
 fired. this event can be used by the GUI framework to set the focus to the widget associated with the new focus flow id.
 
 
@@ -257,7 +258,7 @@ following sub-strings, always in the alphabetic order (like listed below):
 * Shift
 
 the :paramref:`~MainAppBase.key_press_from_framework.key` arg is a string that is specifying the last pressed key. if
-the key is not representing not a single character but a command key, then `key` will be one of the following strings:
+the key is not representing a single character but a command key, then `key` will be one of the following strings:
 
 * escape
 * tab
@@ -311,8 +312,8 @@ these instances within your main app class you can use the attributes :attr:`~Ma
 :attr:`~MainAppBase.framework_root` and :attr:`~MainAppBase.framework_win` of the class :class:`MainAppBase`.
 
 the initialization of the attributes :attr:`~MainAppBase.framework_app`, :attr:`~MainAppBase.framework_root` and
-:attr:`~MainAppBase.framework_win` is optional and can be done e.g. within :meth:`~MainAppBase.init_app` or in the later
-called `on_app_build` application event fired by the framework app instance.
+:attr:`~MainAppBase.framework_win` is optional and can be done e.g. within :meth:`~MainAppBase.init_app` or in the
+`on_app_build` application event fired later by the framework app instance.
 
 .. note::
     if :attr:`~MainAppBase.framework_win` is set to a window instance, then the window instance has to provide a `close`
@@ -392,19 +393,16 @@ from ae.core import DEBUG_LEVELS, registered_app_names
 from ae.console import ConsoleApp                                                                       # type: ignore
 
 
-__version__ = '0.3.83'
+__version__ = '0.3.85'
 
 
-AppStateType = Dict[str, Any]                   #: app state config variable type
-EventKwargsType = Dict[str, Any]                #: change flow event kwargs type
-
 APP_STATE_SECTION_NAME = 'aeAppState'           #: config section name to store app state
 
 #: config variable name to store the current application state version
 APP_STATE_VERSION_VAR_NAME = 'app_state_version'
 
-COLOR_BLACK = (0.009, 0.006, 0.003, 1.0)        # to differentiate from framework pure black/white colors
-COLOR_WHITE = (0.999, 0.996, 0.993, 1.0)
+COLOR_BLACK = [0.009, 0.006, 0.003, 1.0]        #: != 0/1 to differentiate from framework pure black/white colors
+COLOR_WHITE = [0.999, 0.996, 0.993, 1.0]
 THEME_DARK_BACKGROUND_COLOR = COLOR_BLACK       #: dark theme background color in rgba(0.0 ... 1.0)
 THEME_DARK_FONT_COLOR = COLOR_WHITE             #: dark theme font color in rgba(0.0 ... 1.0)
 THEME_LIGHT_BACKGROUND_COLOR = COLOR_WHITE      #: light theme background color in rgba(0.0 ... 1.0)
@@ -434,11 +432,20 @@ HIDDEN_GLOBALS = (
 PORTIONS_IMAGES = FilesRegister()               #: register of image files found in portions/packages at import time
 PORTIONS_SOUNDS = FilesRegister()               #: register of sound files found in portions/packages at import time
 
-USER_NAME_MAX_LEN = 12          #: maximal length of a user name/id (restricting :attr:`~ae.console.ConsoleApp.user_id`)
+USER_NAME_MAX_LEN = 12          #: maximal length of a username/id (restricting :attr:`~ae.console.ConsoleApp.user_id`)
 
 
-if stack_var('__name__') == '__main__':         # only if imported directly from main app module
-    check_all()                                 # .. check/install any outstanding updates or new app versions
+AppStateType = Dict[str, Any]                   #: app state config variable type
+EventKwargsType = Dict[str, Any]                #: change flow event kwargs type
+
+ColorRGB = Union[Tuple[float, float, float], List[float]]           #: color red, green and blue parts
+ColorRGBA = Union[Tuple[float, float, float, float], List[float]]   #: ink is rgb color and alpha
+ColorOrInk = Union[ColorRGB, ColorRGBA]                             #: color or ink type
+
+
+# check/install any outstanding updates or new app versions if imported directly from main app module
+if stack_var('__name__') == '__main__':
+    check_all()                                                                                 # pragma: no cover
 
 
 def ensure_tap_kwargs_refs(init_kwargs: EventKwargsType, tap_widget: Any):
@@ -472,8 +479,8 @@ def ensure_tap_kwargs_refs(init_kwargs: EventKwargsType, tap_widget: Any):
 def flow_action(flow_id: str) -> str:
     """ determine the action string of a flow_id.
 
-    :param flow_id:         flow id.
-    :return:                flow action string.
+    :param flow_id:             flow id.
+    :return:                    flow action string.
     """
     return flow_action_split(flow_id)[0]
 
@@ -481,8 +488,8 @@ def flow_action(flow_id: str) -> str:
 def flow_action_split(flow_id: str) -> Tuple[str, str]:
     """ split flow id string into action part and the rest.
 
-    :param flow_id:         flow id.
-    :return:                tuple of (flow action string, flow obj and key string)
+    :param flow_id:             flow id.
+    :return:                    tuple of (flow action string, flow obj and key string)
     """
     idx = flow_id.find(NAME_PARTS_SEP)
     if idx != -1:
@@ -493,8 +500,8 @@ def flow_action_split(flow_id: str) -> Tuple[str, str]:
 def flow_change_confirmation_event_name(flow_id: str) -> str:
     """ determine the name of the event method for the change confirmation of the passed flow_id.
 
-    :param flow_id:         flow id.
-    :return:                tuple with 2 items containing the flow action and the object name (and id).
+    :param flow_id:             flow id.
+    :return:                    tuple with 2 items containing the flow action and the object name (and id).
     """
     flow, _index = flow_key_split(flow_id)
     action, obj = flow_action_split(flow)
@@ -504,10 +511,10 @@ def flow_change_confirmation_event_name(flow_id: str) -> str:
 def flow_class_name(flow_id: str, name_suffix: str) -> str:
     """ determine class name for the given flow id and class name suffix.
 
-    :param flow_id:         flow id.
-    :param name_suffix:     class name suffix.
-    :return:                name of the class. please note that the flow action `open` will not be added
-                            to the returned class name.
+    :param flow_id:             flow id.
+    :param name_suffix:         class name suffix.
+    :return:                    name of the class. please note that the flow action `open` will not be added
+                                to the returned class name.
     """
     flow, _index = flow_key_split(flow_id)
     action, obj = flow_action_split(flow)
@@ -519,8 +526,8 @@ def flow_class_name(flow_id: str, name_suffix: str) -> str:
 def flow_key(flow_id: str) -> str:
     """ return the key of a flow id.
 
-    :param flow_id:         flow id string.
-    :return:                flow key string.
+    :param flow_id:             flow id string.
+    :return:                    flow key string.
     """
     _action_object, index = flow_key_split(flow_id)
     return index
@@ -529,8 +536,8 @@ def flow_key(flow_id: str) -> str:
 def flow_key_split(flow_id: str) -> Tuple[str, str]:
     """ split flow id into action with object and flow key.
 
-    :param flow_id:         flow id to split.
-    :return:                tuple of (flow action and object string, flow key string).
+    :param flow_id:             flow id to split.
+    :return:                    tuple of (flow action and object string, flow key string).
     """
     idx = flow_id.find(FLOW_KEY_SEP)
     if idx != -1:
@@ -541,14 +548,14 @@ def flow_key_split(flow_id: str) -> Tuple[str, str]:
 def flow_object(flow_id: str) -> str:
     """ determine the object string of the passed flow_id.
 
-    :param flow_id:         flow id.
-    :return:                flow object string.
+    :param flow_id:             flow id.
+    :return:                    flow object string.
     """
     return flow_action_split(flow_key_split(flow_id)[0])[1]
 
 
 def flow_path_id(flow_path: List[str], path_index: int = -1) -> str:
-    """ determine the flow id of the last/newest entry in the flow_path.
+    """ determine the flow id of the newest/last entry in the flow_path.
 
     :param flow_path:           flow path to get the flow id from.
     :param path_index:          index in the flow_path.
@@ -574,9 +581,9 @@ def flow_path_strip(flow_path: List[str]) -> List[str]:
 def flow_popup_class_name(flow_id: str) -> str:
     """ determine name of the Popup class for the given flow id.
 
-    :param flow_id:         flow id.
-    :return:                name of the Popup class. please note that the action `open` will not be added
-                            to the returned class name.
+    :param flow_id:             flow id.
+    :return:                    name of the Popup class. please note that the action `open` will not be added
+                                to the returned class name.
     """
     return flow_class_name(flow_id, 'Popup')
 
@@ -584,10 +591,10 @@ def flow_popup_class_name(flow_id: str) -> str:
 def id_of_flow(action: str, obj: str = "", key: str = "") -> str:
     """ create flow id string.
 
-    :param action:      flow action string.
-    :param obj:         flow object (defined by app project).
-    :param key:         flow index/item_id/field_id/... (defined by app project).
-    :return:            complete flow_id string.
+    :param action:              flow action string.
+    :param obj:                 flow object (defined by app project).
+    :param key:                 flow index/item_id/field_id/... (defined by app project).
+    :return:                    complete flow_id string.
     """
     assert action == '' or FLOW_ACTION_RE.fullmatch(action), \
         f"flow action only allows lowercase letters and digits: got '{action}'"
@@ -603,7 +610,7 @@ def register_package_images():
     """ call from module scope of the package to register/add image/img resources path.
 
     no parameters needed because we use here :func:`~ae.inspector.stack_var` helper function to determine the
-    the module file path via the `__file__` module variable of the caller module in the call stack. in this call
+    module file path via the `__file__` module variable of the caller module in the call stack. in this call
     we have to overwrite the default value (:data:`~ae.inspector.SKIPPED_MODULES`) of the
     :paramref:`~ae.inspector.stack_var.skip_modules` parameter to not skip ae portions that are providing
     package resources and are listed in the :data:`~ae.inspector.SKIPPED_MODULES`, like e.g. :mod:`ae.gui_app` and
@@ -620,7 +627,7 @@ def register_package_sounds():
     """ call from module scope of the package to register/add sound file resources.
 
     no parameters needed because we use here :func:`~ae.inspector.stack_var` helper function to determine the
-    the module file path via the `__file__` module variable of the caller module in the call stack. in this call
+    module file path via the `__file__` module variable of the caller module in the call stack. in this call
     we have to overwrite the default value (:data:`~ae.inspector.SKIPPED_MODULES`) of the
     :paramref:`~ae.inspector.stack_var.skip_modules` parameter to not skip ae portions that are providing
     package resources and are listed in the :data:`~ae.inspector.SKIPPED_MODULES`, like e.g. :mod:`ae.gui_app`
@@ -636,9 +643,9 @@ def register_package_sounds():
 def replace_flow_action(flow_id: str, new_action: str):
     """ replace action in given flow id.
 
-    :param flow_id:         flow id.
-    :param new_action:      action to be set/replaced within passed flow id.
-    :return:                flow id with new action and object/key from passed flow id.
+    :param flow_id:             flow id.
+    :param new_action:          action to be set/replaced within passed flow id.
+    :return:                    flow id with new action and object/key from passed flow id.
     """
     return id_of_flow(new_action, *flow_key_split(flow_action_split(flow_id)[1]))
 
@@ -668,15 +675,15 @@ class MainAppBase(ConsoleApp, ABC):
 
     flow_id: str = ""                                       #: id of the current app flow (entered by the app user)
     flow_path: List[str]                                    #: list of flow ids, reflecting recent user actions
-    flow_id_ink: tuple = (0.99, 0.99, 0.69, 0.69)           #: rgba color for flow id / drag&drop node placeholder
-    flow_path_ink: tuple = (0.99, 0.99, 0.39, 0.48)         #: rgba color for flow_path/drag&drop item placeholder
+    flow_id_ink: ColorOrInk = [0.99, 0.99, 0.69, 0.69]      #: rgba color for flow id / drag&drop node placeholder
+    flow_path_ink: ColorOrInk = [0.99, 0.99, 0.39, 0.48]    #: rgba color for flow_path/drag&drop item placeholder
 
     font_size: float = 21.0                                 #: font size used toolbar and flow screens
     lang_code: str = ""                                     #: optional language code (e.g. 'es_ES' for Spanish)
     light_theme: bool = False                               #: True=light theme/background, False=dark theme
 
-    selected_item_ink: tuple = (0.69, 1.0, 0.39, 0.18)      #: rgba color for list items (selected)
-    unselected_item_ink: tuple = (0.39, 0.39, 0.39, 0.18)   #: rgba color for list items (unselected)
+    selected_item_ink: ColorOrInk = [0.69, 1.0, 0.39, 0.18]  #: rgba color for list items (selected)
+    unselected_item_ink: ColorOrInk = [0.39, 0.39, 0.39, 0.18]  #: rgba color for list items (unselected)
 
     sound_volume: float = 0.12                              #: sound volume of current app (0.0=mute, 1.0=max)
     vibration_volume: float = 0.3                           #: vibration volume of current app (0.0=mute, 1.0=max)
@@ -736,9 +743,9 @@ class MainAppBase(ConsoleApp, ABC):
     def init_app(self, framework_app_class: Any = None) -> Tuple[Optional[Callable], Optional[Callable]]:
         """ initialize framework app instance and root window/layout, return GUI event loop start/stop methods.
 
-        :param framework_app_class:     class to create app instance (optionally extended by app project).
-        :return:                        tuple of two callable, the 1st to start and the 2nd to stop/exit
-                                        the GUI event loop.
+        :param framework_app_class: class to create app instance (optionally extended by app project).
+        :return:                    tuple of two callable, the 1st to start and the 2nd to stop/exit
+                                    the GUI event loop.
         """
 
     def app_state_keys(self) -> Tuple[str, ...]:
@@ -752,7 +759,7 @@ class MainAppBase(ConsoleApp, ABC):
     def backup_config_resources() -> str:   # pragma: no cover
         """ backup config files and image/sound/translations resources to {ado}<now_str>.
 
-        config files are collected from from {ado}, {usr} or {cwd} (the first found file name only - see/sync-with
+        config files are collected from {ado}, {usr} or {cwd} (the first found file name only - see/sync-with
         :meth:`ae.console.ConsoleApp.add_cfg_files`).
 
         resources are copied from {ado} or {cwd} (only the first found resources root path).
@@ -824,7 +831,7 @@ class MainAppBase(ConsoleApp, ABC):
                                 * `changed_event_name`: optional main app event method name to be called if the flow got
                                   confirmed and changed.
                                 * `count`: optional number used to render a pluralized help text for this flow change
-                                  (for this gets also passed to the help text formatter by/in
+                                  (this number gets also passed to the help text formatter by/in
                                   :meth:`~ae.gui_help.HelpAppBase.change_flow`).
                                 * `edit_widget`: optional widget instance for edit/input.
                                 * `flow_id`: process :attr:`~MainAppBase.flow_path` as specified by the
@@ -840,14 +847,14 @@ class MainAppBase(ConsoleApp, ABC):
                                   string value to change the last focus flow id to the passed value.
                                 * `tap_widget`: optional tapped button widget instance (initiating this flow change).
 
-                                some of these keys get passed directly on the call of this method, e.g. via
+                                some of these keys get specified directly on the call of this method, e.g. via
                                 :attr:`~.kivy_app.FlowButton.tap_kwargs` or :attr:`~.kivy_app.FlowToggler.tap_kwargs`,
                                 where others get added by the flow change confirmation handlers/callbacks.
 
         :return:                True if flow got confirmed by a declared custom flow change confirmation event handler
                                 (either event method or Popup class) of the app and changed accordingly, else False.
 
-                                some flow actions are handled internally independent from the return value of a
+                                some flow actions are handled internally independent of the return value of a
                                 custom event handler, like e.g. `'enter'` or `'leave'` will always extend or reduce the
                                 flow path and the action `'focus'` will give the indexed widget the input focus (these
                                 exceptions are configurable via :data:`ACTIONS_CHANGING_FLOW_WITHOUT_CONFIRMATION`).
@@ -920,7 +927,7 @@ class MainAppBase(ConsoleApp, ABC):
     def class_by_name(class_name: str) -> Optional[Type]:
         """ search class name in framework modules as well as in app main.py to return class object.
 
-        dirty fallback using the inspect module - implemented as method to allow overwrite in your app project.
+        fallback using the :mod:`ae.inspector` module - overwrite in your app project to customize.
 
         :param class_name:      name of the class.
         :return:                class object with the specified class name or :data:`~ae.base.UNSET` if not found.
@@ -994,7 +1001,7 @@ class MainAppBase(ConsoleApp, ABC):
         return child_wid(self.widget_children(self.framework_win))
 
     def flow_path_action(self, flow_path: Optional[List[str]] = None, path_index: int = -1) -> str:
-        """ determine the action of the last/newest entry in the flow_path.
+        """ determine the action of the last (newest) entry in the flow_path.
 
         :param flow_path:       optional flow path to get the flow action from (default=self.flow_path).
         :param path_index:      optional index in the flow_path (default=-1).
@@ -1104,8 +1111,8 @@ class MainAppBase(ConsoleApp, ABC):
 
     def mix_background_ink(self):
         """ remix background ink if one of the basic back colours change. """
-        self.framework_app.mixed_back_ink = (sum(_) / len(_) for _ in zip(
-            self.flow_id_ink, self.flow_path_ink, self.selected_item_ink, self.unselected_item_ink))
+        self.framework_app.mixed_back_ink = [sum(_) / len(_) for _ in zip(
+            self.flow_id_ink, self.flow_path_ink, self.selected_item_ink, self.unselected_item_ink)]
 
     def on_app_build(self):
         """ default/fallback flow change confirmation event handler. """
@@ -1229,7 +1236,7 @@ class MainAppBase(ConsoleApp, ABC):
         """ called on close of UserNameEditorPopup to check user input and create/register the current os user.
 
         :param user_name:       new user id.
-        :param event_kwargs:    optionally pass True in the `unique_user_name` key to prevent duplicate user names.
+        :param event_kwargs:    optionally pass True in the `unique_user_name` key to prevent duplicate usernames.
         :return:                True if user got registered else False.
         """
         if not user_name:
@@ -1299,7 +1306,7 @@ class MainAppBase(ConsoleApp, ABC):
         self.dpo(f"MainAppBase.open_popup {popup_class} {popup_kwargs}")
         popup_instance = popup_class(**popup_kwargs)
         open_method = getattr(popup_instance, 'open', getattr(popup_instance, 'show', None))
-        if open_method:
+        if callable(open_method):
             open_method()
         return popup_instance
 
@@ -1547,7 +1554,7 @@ class MainAppBase(ConsoleApp, ABC):
                     getattr(wid, 'opacity', True))
 
     def win_pos_size_change(self, *win_pos_size):
-        """ screen resize handler (called on window resize or when app will exit/stop via closed event.
+        """ screen resize handler called on window resize or when app will exit/stop via closed event.
 
         :param win_pos_size:    window geometry/coordinates: x, y, width, height.
         """
